//+------------------------------------------------------------------+
//|                                                   Bago_AHA.mq4   |
//|                                   Copyright ?2006, Hua Ai (aha)  |
//+------------------------------------------------------------------+

/*
  +------------------------------------------------------------------+
  | Generate raw signals based on ema and RSI crosses, abstract real |
  | signals from them based on vegas tunnel, its fibo lines, as well |
  | as how close the ema and RSI crosses happen.                     |
  | Use vegas tunnel and its fibo lines for initial exit. When price |
  | reach high fibo lines, change to trailing stop.                  |
  +------------------------------------------------------------------+
*/   
#property copyright "Copyright ?2006, Hua Ai(aha)"
#property link      ""

#property indicator_chart_window
#property indicator_buffers 4
#property indicator_color1 Green
#property indicator_color2 Red
#property indicator_color3 Green
#property indicator_color4 Red

#property indicator_width1 2
#property indicator_width2 2
#property indicator_width3 2
#property indicator_width4 2

//Adjustable parameters
extern int     TrailingStop        =  30;
extern int     CrossEffectiveTime  =  2; 
extern int     TunnelBandWidth     =  0;
extern int     TunnelSafeZone      =  120;
extern int     HardStopLoss        =  30;

extern bool    LondonOpen          =  true;
extern bool    NewYorkOpen         =  true;
extern bool    TokyoOpen           =  true;

extern int     FasterEMA           =  5;
extern int     SlowerEMA           =  12;
extern int     RSIPeriod           =  21;
extern bool    Alerts              =  true;

extern double  TotalLots           =  3.0;
extern int     TPLevel1            =  55;
extern double  TPLevel1Lots        =  1.0;
extern int     TPLevel2            =  89;
extern double  TPLevel2Lots        =  1.0;
extern int     TPLevel3            =  144;

// Indicator buffers to mark the signal
double         CrossUp[];
double         CrossDown[];
double         LongDanger[];
double         ShortDanger[];

// State registers store cross up/down information 
bool           EMACrossedUp;
bool           EMACrossedDown;
bool           RSICrossedUp;
bool           RSICrossedDown;
bool           TunnelCrossedUp;
bool           TunnelCrossedDown;

// Cross up/down info should expire in a couple of bars.
// Timer registers to control the expiration.
int            EMACrossedUpTimer;
int            RSICrossedUpTimer;
int            EMACrossedDownTimer;
int            RSICrossedDownTimer;

// Registers for visual alerts on charts.
bool           startup;
bool           upalert,downalert;
int            SignalLabeled, DangerLabeled; // 0: initial state; 1: up; 2: down.
//+------------------------------------------------------------------+
//| Custom indicator initialization function                         |
//+------------------------------------------------------------------+
int init()
  {
//---- indicators
   SetIndexStyle(0, DRAW_ARROW, EMPTY);
   SetIndexArrow(0, 241);
   SetIndexBuffer(0, CrossUp);
   SetIndexStyle(1, DRAW_ARROW, EMPTY);
   SetIndexArrow(1, 242);
   SetIndexBuffer(1, CrossDown);

   SetIndexStyle(2, DRAW_ARROW, EMPTY);
   SetIndexArrow(2, 251);
   SetIndexBuffer(2, LongDanger);
   SetIndexStyle(3, DRAW_ARROW, EMPTY);
   SetIndexArrow(3, 251);
   SetIndexBuffer(3, ShortDanger);
//----
   // Initialize cross state registers
   EMACrossedUp      = false;
   RSICrossedUp      = false;
   TunnelCrossedUp   = false;
   EMACrossedDown    = false;
   RSICrossedDown    = false;
   TunnelCrossedDown = false;
   
   // Reset timers
   EMACrossedUpTimer = 0;
   RSICrossedUpTimer = 0;
   EMACrossedDownTimer = 0;
   RSICrossedDownTimer = 0;

   // Initialize visual alert registers
   startup           = true;
   upalert           = false;
   downalert         = false;
   SignalLabeled     = 0;
   DangerLabeled     = 0;
   
   return(0);
  }
//+------------------------------------------------------------------+
//| Custom indicator deinitialization function                       |
//+------------------------------------------------------------------+
int deinit()
  {
//---- 
   // Initialize cross state registers
   EMACrossedUp      = false;
   RSICrossedUp      = false;
   TunnelCrossedUp   = false;
   EMACrossedDown    = false;
   RSICrossedDown    = false;
   TunnelCrossedDown = false;
   
   // Reset timers
   EMACrossedUpTimer = 0;
   RSICrossedUpTimer = 0;
   EMACrossedDownTimer = 0;
   RSICrossedDownTimer = 0;

   // Initialize visual alert registers
   startup           = true;
   upalert           = false;
   downalert         = false;
   SignalLabeled     = 0;
   DangerLabeled     = 0;
//----
   return(0);
  }
//+------------------------------------------------------------------+
//| Custom indicator iteration function                              |
//+------------------------------------------------------------------+
int start() {
   int    limit, i, counter;
   double fasterEMAnow, slowerEMAnow;
   double fasterEMAprevious, slowerEMAprevious;
   double RSInow, RSIprevious;
   double VegasTunnelFast, VegasTunnelSlow;
   double VegasPriceCrossedL, VegasPriceCrossedS;
   double Range, AvgRange;

   int counted_bars=IndicatorCounted();
   if (counted_bars<0) return(-1);      // check for possible errors
   if (counted_bars>0) counted_bars--;  // last counted bar will be recounted

   limit=Bars-counted_bars;
   
   // Check the condition at the bar close.
   if ((Minute()==58 && Seconds()>50) || startup)
   { 
      for(i = limit; i >= 0; i--) {
   
         // Calculate the range of recent two bars.
         // The result will be used to locate where the signal marks is put.
         counter=i;
         Range=0;
         AvgRange=0;
         for (counter=i ;counter<=i+1;counter++)
         {
            AvgRange=AvgRange+MathAbs(High[counter]-Low[counter]);
         }
         Range=AvgRange/2;
         
         // *********************************************************************
         // Data sampling and calculation
         // *********************************************************************
          
         fasterEMAnow      = iMA(NULL, 0, FasterEMA, 0, MODE_EMA, PRICE_CLOSE, i);
         fasterEMAprevious = iMA(NULL, 0, FasterEMA, 0, MODE_EMA, PRICE_CLOSE, i+1);

         slowerEMAnow      = iMA(NULL, 0, SlowerEMA, 0, MODE_EMA, PRICE_CLOSE, i);
         slowerEMAprevious = iMA(NULL, 0, SlowerEMA, 0, MODE_EMA, PRICE_CLOSE, i+1);
      
         RSInow            = iRSI(NULL,0,RSIPeriod,PRICE_CLOSE,i);
         RSIprevious       = iRSI(NULL,0,RSIPeriod,PRICE_CLOSE,i+1);
      
         VegasTunnelFast   = iMA(NULL,0,144,0,1,PRICE_CLOSE,i);
         VegasTunnelSlow   = iMA(NULL,0,169,0,1,PRICE_CLOSE,i);
         
         // *********************************************************************
         // Based on the price and the calculated indicators determine the states
         // of the state machine.
         // *********************************************************************

         // Check if there is a RSI cross up
         if ((RSInow > 50) && (RSIprevious < 50) && (RSICrossedUp == false)) 
         {
            RSICrossedUp   = true;
            RSICrossedDown = false;
         }
         
         if (RSICrossedUp == true) 
            RSICrossedUpTimer++;  // Record the number of bars the cross has happened
         else
            RSICrossedUpTimer=0;  // Reset the timer once the cross is reversed
         
         if (RSICrossedUpTimer>=CrossEffectiveTime) 
            RSICrossedUp = false; // Reset state register when crossed 3 bars ago
   
         // Check if there is a RSI cross down
         if ((RSInow < 50) && (RSIprevious > 50) && (RSICrossedDown == false)) 
         {
            RSICrossedUp   = false;
            RSICrossedDown = true;
         }

         if (RSICrossedDown == true) 
            RSICrossedDownTimer++;  // Record the number of bars the cross has happened
         else
            RSICrossedDownTimer=0;  // Reset the timer once the cross is reversed
         
         if (RSICrossedDownTimer>=CrossEffectiveTime) 
            RSICrossedDown = false; // Reset register when crossed 3 bars ago

         // Check if there is a EMA cross up
         if ( (fasterEMAnow > slowerEMAnow) && 
              (fasterEMAprevious < slowerEMAprevious) && 
              (EMACrossedUp == false) ) 
         {
            EMACrossedUp   = true;
            EMACrossedDown = false;
         }

         if (EMACrossedUp == true) 
            EMACrossedUpTimer++;  // Record the number of bars the cross has happened
         else
            EMACrossedUpTimer=0;  // Reset the timer once the cross is reversed
         
         if (EMACrossedUpTimer>=CrossEffectiveTime) 
            EMACrossedUp = false; // Reset register when crossed 3 bars ago

         // Check if there is a EMA cross down
         if ( (fasterEMAnow < slowerEMAnow) && 
              (fasterEMAprevious > slowerEMAprevious) && 
              (EMACrossedDown == false) ) 
         {
            EMACrossedUp   = false;
            EMACrossedDown = true;
         }

         if (EMACrossedDown == true) 
            EMACrossedDownTimer++;  // Record the number of bars the cross has happened
         else
            EMACrossedDownTimer=0;  // Reset the timer once the cross is reversed
         
         if (EMACrossedDownTimer>=CrossEffectiveTime) 
            EMACrossedDown = false; // Reset register when crossed 3 bars ago

         
         if ((Close[1]>VegasTunnelFast && Close[1]>VegasTunnelSlow) && 
             (Close[2]<VegasTunnelFast || Close[2]<VegasTunnelSlow))
         {
            TunnelCrossedUp   = true;
            TunnelCrossedDown = false;
         }

         if ((Close[1]<VegasTunnelFast && Close[1]<VegasTunnelSlow) && 
             (Close[2]>VegasTunnelFast || Close[2]>VegasTunnelSlow))
         {
            TunnelCrossedUp   = false;
            TunnelCrossedDown = true;
         }

         // *********************************************************************
         // Based on states, determine entry situations.
         // *********************************************************************
         
         if (  // Raw long signal generating rules
               (EMACrossedUp == true) && (RSICrossedUp == true) && (SignalLabeled != 1) &&
               
               // Experimenting new filters here
               // 1. Should only enter long on the top of the tunnel to avoid tunnel resistance
               // 2. Shouldn't enter long on the high fibo lines above tunnel because the price  
               //    tends to go back to tunnel.
               // 3. Instead of push up from the tunnel, the price may drop from high fibo lines  
               //    but with EMA and RSI crossed, should avoid enter long at such situation.
               // 4. Around tunnel there ususually have a lot whipsaw, should avoid enter a trade
               //    in the band.
               // 5. Should allow the price push from below the tunnel and enter long trade.
               // 6. Only trade on high volatile market hours.
               ( (Close[i] >= VegasTunnelSlow+TunnelBandWidth*Point && // Above whipsaw zone
                  Close[i] <= VegasTunnelSlow+TunnelSafeZone*Point  && // Below high fibo line
                  Open[i]  <= Close[i]) ||                             // Price push up from a bull bar
                 (Close[i] <= VegasTunnelSlow-TunnelBandWidth*Point)&& // Testing from below the tunnel
                 ( (LondonOpen==true  && Hour()>=7 && Hour()<=16)  ||
                   (NewYorkOpen==true && Hour()>=12 && Hour()<=21) ||
                   (TokyoOpen==true   && ((Hour()>=0 && Hour()<=8) || 
                                          (Hour()>=23 && Hour()<=24)))
                 )
               )
            ) 
         {
            CrossUp[i] = Low[i] - Range*0.3;
            
            // Signal checkup happens every tick after the 50'50", once a signal is triggered, the program
            // should alert even though the bar haven't closed yet. The informed trader should wait until 
            // the bar close to make decision. In another word, even though the conditions to trigger the 
            // signal could disappear at the end of the bar, the alert will still be sent to draw trader's
            // attention.

            // The signal will be validated at the beginning of the next bar, at 5'50".

            // !upalert and !downalert are added to prevent multiple alerts in the same bar. The price usually 
            // fluctuates at the end of each bar and it may trigger signals multiple times, the trade only 
            // needs to be informed once. Once a signall is triggered, the alert will stay true to block the 
            // subsequent signals. It will be cleared at the validation process at the 5'50" of the next bar.
            
            if(i<=2 && Alerts && !upalert && !downalert && !startup) { 
               Alert (Symbol()," ",Period(),"M  EMACross UP ");
               SendMail("EMA Cross Up on "+Symbol(),"Buy at "+Ask);
               upalert=true;
               //downalert=false;
            }
            SignalLabeled = 1;
            DangerLabeled = 0;
         }

         else if 
            (  (EMACrossedDown == true) && (RSICrossedDown == true) && (SignalLabeled != 2) && 
               // Similar to long signal filtering rules.
               ( (Close[i] <= VegasTunnelSlow-TunnelBandWidth*Point && // Below whipsaw zone
                  Close[i] >= VegasTunnelSlow-TunnelSafeZone*Point  && // Above high fibo line
                  Open[i]  >  Close[i]) ||                             // Price down from a bear bar
                 (Close[i] >= VegasTunnelSlow+TunnelBandWidth*Point)&& // Testing from above the tunnel
                 ( (LondonOpen==true  && Hour()>=7 && Hour()<=16)  ||
                   (NewYorkOpen==true && Hour()>=12 && Hour()<=21) ||
                   (TokyoOpen==true   && ((Hour()>=0 && Hour()<=8) || 
                                          (Hour()>=23 && Hour()<=24)))
                 )
               )
            )
         {
            CrossDown[i] = High[i] + Range*0.3;
            if(i<=2 && Alerts && !upalert && !downalert && !startup) {
               Alert (Symbol()," ",Period(),"M  EMACross DOWN ");
               SendMail("EMA Cross Down on "+Symbol(),"Sell at "+Bid);
               downalert=true;
               //upalert=false;
            }
            SignalLabeled = 2;
            DangerLabeled = 0;
         }
         
         // *********************************************************************
         // Based on states, determine dangerous situations.
         // *********************************************************************
         if ( (SignalLabeled == 1) && 
              (EMACrossedDown==true || RSICrossedDown==true) && 
              (DangerLabeled != 1)
            ) 
         {
            LongDanger[i] = High[i] + Range*0.2;
            DangerLabeled = 1;
            SignalLabeled = 0;
         }

         if ( (SignalLabeled == 2) && 
              (EMACrossedUp==true || RSICrossedUp==true) && 
              (DangerLabeled != 2)
            )
         {
            ShortDanger[i] = Low[i] - Range*0.2;
            DangerLabeled  = 2;
            SignalLabeled = 0;
         }
      }
   }

   if ((upalert || downalert) && Minute()==5 && Seconds()>50)
   {

      //Validate the alert after the last bar close.

         fasterEMAnow      = iMA(NULL, 0, FasterEMA, 0, MODE_EMA, PRICE_CLOSE, 1);
         fasterEMAprevious = iMA(NULL, 0, FasterEMA, 0, MODE_EMA, PRICE_CLOSE, 2);

         slowerEMAnow      = iMA(NULL, 0, SlowerEMA, 0, MODE_EMA, PRICE_CLOSE, 1);
         slowerEMAprevious = iMA(NULL, 0, SlowerEMA, 0, MODE_EMA, PRICE_CLOSE, 2);
      
         RSInow      = iRSI(NULL,0,RSIPeriod,PRICE_CLOSE,1);
         RSIprevious = iRSI(NULL,0,RSIPeriod,PRICE_CLOSE,2);

         if (upalert)
         {
            if ((RSInow > 50) && (fasterEMAnow > slowerEMAnow))
            {
               //validated.
            }
            else
            {
               CrossUp[1] = 0;
               
               // Restore the value of the state machine registers.
               // Unlike the signal check up part, we only need to restore the states and don't care 
               // about when the crosses happen. So we only need to check the current value.
               if (RSInow < 50)
               {
                  RSICrossedUp   = false;
                  RSICrossedDown = true;
               }

               if (fasterEMAnow < slowerEMAnow) {
                  EMACrossedUp   = false;
                  EMACrossedDown = true;
               }

               SignalLabeled = 2;
               DangerLabeled = 0;
            }
         }      

         if (downalert)
         {
            if ((RSInow < 50) && (fasterEMAnow < slowerEMAnow))
            {
               //validated.
            } 
            else 
            {
               CrossDown[1] = 0;
               
               // Restore the value of the state machine registers.
               // Unlike the signal check up part, we only need to restore the states and don't care 
               // about when the crosses happen. So we only need to check the current value.
               if (RSInow > 50)
               {
                  RSICrossedUp   = true;
                  RSICrossedDown = false;
               }

               if (fasterEMAnow > slowerEMAnow) {
                  EMACrossedUp   = true;
                  EMACrossedDown = false;
               }

               SignalLabeled = 1;
               DangerLabeled = 0;
            }
         }

      //Validation ends here.

      downalert = false;
      upalert   = false;

   }

   startup = false;
   return(0);
}

